import { useNuxt, defineNuxtModule, createResolver, useLogger, addImportsDir } from '@nuxt/kit';
import { readPackageJSON } from 'pkg-types';
import { existsSync } from 'node:fs';

const DEVTOOLS_UI_ROUTE = "/__nuxt-scripts";
const DEVTOOLS_UI_LOCAL_PORT = 3030;
function setupDevToolsUI(options, resolve, nuxt = useNuxt()) {
  const clientPath = resolve("./client");
  const isProductionBuild = existsSync(clientPath);
  if (isProductionBuild) {
    nuxt.hook("vite:serverCreated", async (server) => {
      const sirv = await import('sirv').then((r) => r.default || r);
      server.middlewares.use(
        DEVTOOLS_UI_ROUTE,
        sirv(clientPath, { dev: true, single: true })
      );
    });
  } else {
    nuxt.hook("vite:extendConfig", (config) => {
      config.server = config.server || {};
      config.server.proxy = config.server.proxy || {};
      config.server.proxy[DEVTOOLS_UI_ROUTE] = {
        target: `http://localhost:${DEVTOOLS_UI_LOCAL_PORT}${DEVTOOLS_UI_ROUTE}`,
        changeOrigin: true,
        followRedirects: true,
        rewrite: (path) => path.replace(DEVTOOLS_UI_ROUTE, "")
      };
    });
  }
  nuxt.hook("devtools:customTabs", (tabs) => {
    tabs.push({
      // unique identifier
      name: "nuxt-scripts",
      // title to display in the tab
      title: "Scripts",
      // any icon from Iconify, or a URL to an image
      icon: "carbon:script",
      // iframe view
      view: {
        type: "iframe",
        src: DEVTOOLS_UI_ROUTE
      }
    });
  });
}

const module = defineNuxtModule({
  meta: {
    name: "@nuxt/scripts",
    configKey: "scripts",
    compatibility: {
      nuxt: "^3.9.0",
      bridge: false
    }
  },
  defaults: {
    enabled: true,
    debug: false
  },
  async setup(config, nuxt) {
    const { resolve } = createResolver(import.meta.url);
    const { name, version } = await readPackageJSON(resolve("../package.json"));
    const logger = useLogger(name);
    if (config.enabled === false) {
      logger.debug("The module is disabled, skipping setup.");
      return;
    }
    nuxt.options.alias["#nuxt-scripts"] = resolve("./runtime/types");
    nuxt.options.runtimeConfig["nuxt-scripts"] = { version };
    nuxt.options.runtimeConfig.public["nuxt-scripts"] = { defaults: config.defaults };
    addImportsDir(resolve("./runtime/composables"));
    if (nuxt.options.dev)
      setupDevToolsUI(config, resolve);
  }
});

export { module as default };
